from django.core.management.base import BaseCommand

from authanon.backends import display_permissions


class Command(BaseCommand):
    help = "Shows the permissions for anonymous and logged in users. Groups will be created if they do not yet exist."

    def handle(self, *args, **options):
        display_permissions()



from django.contrib.auth.models import Group
from django.contrib.auth.backends import BaseBackend
from django.conf import settings


def get_group_from_settings(key, default):
    group_name = getattr(settings, key, default)
    group, _ = Group.objects.get_or_create(name=group_name)
    return group


def get_anonymous_group():
    return get_group_from_settings("AUTHANON_ANONYMOUS_GROUP", "Anonymous")


def get_login_group():
    return get_group_from_settings("AUTHANON_LOGIN_GROUP", "Login Users")


def permissions_for_group(group):
    # largely adapted from django.contrib.auth.backends
    perms = group.permissions.all()
    perms = perms.values_list("content_type__app_label", "codename").order_by()
    perms = set(f"{ct}.{codename}" for ct, codename in perms)
    return perms


def get_anonymous_group_permissions():
    return permissions_for_group(get_anonymous_group())


def get_login_group_permissions():
    return permissions_for_group(get_login_group())


def display_permissions_group(group_description, group):
    print(f"{group_description} ('{group.name}') Permissions:")
    perms = permissions_for_group(group)
    if not perms:
        perms = ("None",)
    for perm in perms:
        print("\t", perm)


def display_permissions():
    display_permissions_group("Anonymous Group", get_anonymous_group())
    display_permissions_group("Login Group", get_login_group())


class AuthanonBackend(BaseBackend):
    """Adapted from https://stackoverflow.com/a/31520798"""

    perm_cache_name = "_authanon_perm_cache"

    def get_group_permissions(self, user_obj, obj=None):
        if not hasattr(user_obj, self.perm_cache_name):
            perms = get_anonymous_group_permissions()
            if not user_obj.is_anonymous:
                perms |= get_login_group_permissions()
            setattr(user_obj, self.perm_cache_name, perms)
        return getattr(user_obj, self.perm_cache_name)

    def get_all_permissions(self, user_obj, obj=None):
        return self.get_group_permissions(user_obj, obj)

    def has_perm(self, user_obj, perm, obj=None):
        return perm in self.get_all_permissions(user_obj, obj)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['authanon', 'authanon.management.commands', 'authanon.migrations']

package_data = \
{'': ['*']}

install_requires = \
['Django>=3.0,<4.0']

setup_kwargs = {
    'name': 'django-authanon',
    'version': '0.1.2',
    'description': 'Allows permissions for an anonymous user and a generic signed-in user to be set as groups.',
    'long_description': '# django-authanon\n\n![pipline](https://github.com/rbturnbull/django-authanon/actions/workflows/pipeline.yml/badge.svg)\n[<img src="https://img.shields.io/endpoint?url=https://gist.githubusercontent.com/rbturnbull/49262550cc8b0fb671d46df58de213d4/raw/django-authanon-coverage.json">](<https://rbturnbull.github.io/django-authanon/>)\n[<img src="https://img.shields.io/badge/code%20style-black-000000.svg">](<https://github.com/psf/black>)\n\nAllows permissions for an anonymous user and a generic signed-in user to be set as groups.\n\n## Installation \n\n```\npip install django-authanon\n```\n\nThen add to your `INSTALLED_APPS`:\n```\nINSTALLED_APPS += [\n    "authanon",\n]\n```\n\nThen add to your `AUTHENTICATION_BACKENDS`:\n```\nAUTHENTICATION_BACKENDS += [\n    "authanon.backends.AuthanonBackend",\n]\n```\n\n## Usage\n\nThis app creates two groups, one for anonymous users who aren\'t logged in and one group for users who are logged in. You can add permissions to these groups in the admin console in the \'Groups\' section under the \'AUTHENTICATION AND AUTHORIZATION\' section.\n\nTo display the permissions for these two groups on the command line, use this command.\n```\n./manage.py authanon\n```\n\nThese groups are automatically created when anonymous users or logged-in users try to access pages. If you find they haven\'t been created yet, use the `./manage.py authanon` command and then the groups will appear in the admin.\n\n## Configuration\nBy default, the two groups are called `Anonymous` and `Login Users`. You can change them by variables to the settings with the names `AUTHANON_ANONYMOUS_GROUP` or `AUTHANON_LOGIN_GROUP`.\n\n\n## Credits\nPackage authored by Robert Turnbull (Melbourne Data Analytics Platform)\n\nInspired by this Stack Overflow answer: https://stackoverflow.com/a/31520798 (User: jozxyqk)\n',
    'author': 'Robert Turnbull',
    'author_email': 'robert.turnbull@unimelb.edu.au',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/rbturnbull/django-authanon',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.1,<4.0.0',
}


setup(**setup_kwargs)
